/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.secure;

import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;
import org.wso2.usermanager.readwrite.DefaultRealm;
import org.wso2.usermanager.readwrite.DefaultRealmConfig;
import org.wso2.registry.RegistryException;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.ActionConstants;
import org.wso2.registry.jdbc.DatabaseConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

public class RegistryRealmFactory {

    private static final Log log = LogFactory.getLog(RegistryRealmFactory.class);

    public Realm newInMemoryRegistryRealm(String sqlFilePath) throws RegistryException {

        new HSQLDBInitializer().createHSQLTables();
        return newRegistryRealm(sqlFilePath,
                UserManagerConstants.HSQL_DB_URL, DatabaseConstants.HSQL_DRIVER_NAME);
    }

    public Realm newRegistryRealm(String sqlFilePath, String dbURL, String driver)
            throws RegistryException {

        try {            
            
            DefaultRealm realm = new DefaultRealm();
            DefaultRealmConfig config = (DefaultRealmConfig)realm.getRealmConfiguration();
            config.setSqlFile(sqlFilePath);
            config.setConnectionURL(dbURL);
            config.setDriverName(driver);
            realm.init(config);

            String[] users = realm.getAllUserNames();
            boolean adminDefined = false;
            boolean anonymousDefined = false;
            for (int i = 0; i < users.length; i++) {
                if (users[i] != null) {
                    if (users[i].equals(RegistryConstants.ADMIN_USER)) {
                        adminDefined = true;
                    } else if (users[i].equals(RegistryConstants.ANONYMOUS_USER)) {
                        anonymousDefined = true;
                    }
                }
            }

            if (!adminDefined) {
                realm.addUser(RegistryConstants.ADMIN_USER, "admin");

                realm.authorizeUser(RegistryConstants.ADMIN_USER, RegistryConstants.ROOT_PATH, ActionConstants.GET);
                realm.authorizeUser(RegistryConstants.ADMIN_USER, RegistryConstants.ROOT_PATH, ActionConstants.PUT);
                realm.authorizeUser(RegistryConstants.ADMIN_USER, RegistryConstants.ROOT_PATH, ActionConstants.DELETE);
                realm.authorizeUser(RegistryConstants.ADMIN_USER, RegistryConstants.ROOT_PATH, ActionConstants.AUTHORIZE);

                // authorizations for registry level actions are set in the root path
                realm.authorizeUser(RegistryConstants.ADMIN_USER, RegistryConstants.ROOT_PATH, ActionConstants.ADD_USER);
                realm.authorizeUser(RegistryConstants.ADMIN_USER, RegistryConstants.ROOT_PATH, ActionConstants.REMOVE_USER);
            }

            if (!anonymousDefined) {
                realm.addUser(RegistryConstants.ANONYMOUS_USER, "guest");
            }

            return realm;

        } catch (Exception e) {
            e.printStackTrace();
            throw new RegistryException(e.getMessage());
        }
    }
}
